using System;
using Benday.Npr.Common.Repositories;
using Benday.Npr.Common;
using Benday.Npr.Common.DomainModel;
using System.Net;
using System.IO;

namespace Benday.Npr.DataAccess
{
    public class StoryRepository : IStoryRepository
    {
        public StoryRepository(IObjectFactory factory)
        {
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        private IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }

        public void GetStories(ReturnResult<IStoryList> action, string id)
        {
            if (String.IsNullOrEmpty(id))
                throw new ArgumentException("id is null or empty.", "id");
            if (action == null)
                throw new ArgumentNullException("action", "action is null.");

            if (id.StartsWith("RSS_FEED.") == true)
            {
                // RSS_FEED.ITS_ALL_POLITICS
                // RSS_FEED.NPR_HOURLY_NEWS
                GetStoriesForRssFeed(action, id);
            }
            else
            {
                GetStoriesForProgram(action, id);
            }
        }

        private void GetStoriesForRssFeed(ReturnResult<IStoryList> action, string id)
        {
            string url;
            
            if (id == "RSS_FEED.NPR_HOURLY_NEWS")
            {
                url = "http://www.npr.org/rss/podcast.php?id=500005";
            }
            else if (id == "RSS_FEED.ITS_ALL_POLITICS")
            {
                url = "http://www.npr.org/rss/podcast.php?id=510068";
            }
            else if (id == "RSS_FEED.CAR_TALK")
            {
                url = "http://www.npr.org/rss/podcast.php?id=510208";
            }
            else
            {
                throw new InvalidOperationException(String.Format("Unknown rss feed id '{0}'.", id));
            }

            HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;

            request.BeginGetResponse(ProcessStoriesFromRssFeedResult, new WebRequestReturnResult<IStoryList>(request, action));
        }

        private void ProcessStoriesFromRssFeedResult(IAsyncResult asyncResult)
        {
            var action = asyncResult.AsyncState as WebRequestReturnResult<IStoryList>;

            if (action == null)
            {
                throw new InvalidOperationException("User state was the wrong type.");
            }

            try
            {
                var response = action.Request.EndGetResponse(asyncResult);

                using (var stream = new StreamReader(response.GetResponseStream()))
                {
                    string xml = stream.ReadToEnd();

                    var adapter = ObjectFactoryInstance.Create<IRssFeedXmlAdapter>();

                    var returnValue = ObjectFactoryInstance.Create<IStoryList>();

                    adapter.Adapt(xml, returnValue);

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(returnValue));
                }
            }
            catch (Exception)
            {
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(null as IStoryList));
            }  
        }        

        private void GetStoriesForProgram(ReturnResult<IStoryList> action, string id)
        {
            string url = String.Format("http://api.npr.org/query?id={0}&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001", id);

            HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;

            request.BeginGetResponse(ProcessStoriesFromQueryResult, new WebRequestReturnResult<IStoryList>(request, action));
        }

        private void ProcessStoriesFromQueryResult(IAsyncResult asyncResult)
        {
            var action = asyncResult.AsyncState as WebRequestReturnResult<IStoryList>;

            if (action == null)
            {
                throw new InvalidOperationException("User state was the wrong type.");
            }

            try
            {
                var response = action.Request.EndGetResponse(asyncResult);

                using (var stream = new StreamReader(response.GetResponseStream()))
                {
                    string xml = stream.ReadToEnd();

                    var adapter = ObjectFactoryInstance.Create<IStoryXmlAdapter>();

                    var returnValue = ObjectFactoryInstance.Create<IStoryList>();

                    adapter.Adapt(xml, returnValue);

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(returnValue));
                }
            }
            catch (Exception)
            {
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(null as IStoryList));
            }  
        }

        private void ProcessMp3ResolveResult(IAsyncResult asyncResult)
        {
            var action = asyncResult.AsyncState as WebRequestReturnResult<string>;

            if (action == null)
            {
                throw new InvalidOperationException("User state was the wrong type.");
            }

            try
            {
                var response = action.Request.EndGetResponse(asyncResult);

                using (var stream = new StreamReader(response.GetResponseStream()))
                {
                    string m3uPlaylistContents = stream.ReadToEnd();

                    StringReader playlistReader = new StringReader(m3uPlaylistContents);

                    string urlToMp3File = playlistReader.ReadLine();

                    while (urlToMp3File != null && urlToMp3File.StartsWith("http://") == false)
                    {
                        urlToMp3File = playlistReader.ReadLine();
                    }

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(urlToMp3File));
                }   
            }
            catch (Exception ex)
            {
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(ex));                
            }            
        }

        public void ResolveMp3UrlFromM3u(ReturnResult<string> action, string m3uUri)
        {
            if (System.String.IsNullOrEmpty(m3uUri))
                throw new ArgumentNullException("m3uUri", "m3uUri is null.");
            if (action == null)
                throw new ArgumentNullException("action", "action is null.");

            try
            {
                HttpWebRequest request = WebRequest.Create(m3uUri) as HttpWebRequest;

                request.BeginGetResponse(ProcessMp3ResolveResult, new WebRequestReturnResult<string>(request, action));    
            }
            catch (Exception)
            {
                throw;   
            }
        }
    }
}
