using System;
using GalaSoft.MvvmLight;
using System.Windows.Input;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using Benday.Npr.Common;
using Benday.Npr.Presentation.Interfaces;
using Benday.Npr.Common.DomainModel;
using System.Windows;

namespace Benday.Npr.Presentation
{
    public class StoryListViewModel : ObjectFactoryViewModel, IStoryListViewModel
    {
        public StoryListViewModel(IStoryDomainService service, IObjectFactory factory)
            : base(factory)
        {
            if (service == null)
                throw new ArgumentNullException("service", "service is null.");

            m_Stories = new ObservableCollection<IStoryViewModel>();

            m_ServiceInstance = service;
        }

        private IStoryDomainService m_ServiceInstance;
        private IStoryDomainService ServiceInstance
        {
            get { return m_ServiceInstance; }
        }

        public void Load(string programId)
        {
            IsProgressBarVisible = true;
            ProgressBarMessage = "Loading stories..."; 
            
            Id = programId;
            ServiceInstance.GetStories(new ReturnResult<IStoryList>(StoryListReturned), programId);
        }

        private void StoryListReturned(ReturnResult<IStoryList> result)
        {
            if (result.Error == null)
            {
                Populate(result.Result);
            }
            else
            {
                IsProgressBarVisible = false;

                MessageBox.Show("Sorry.  We had some problems getting the stories.  (shrug)");
            }
        }

        private void Populate(IStoryList fromValues)
        {
            ProgressBarMessage = "Parsing stories...";

            try
            {
                this.Stories.Clear();

                if (fromValues == null)
                {
                    Title = String.Empty;
                    Teaser = String.Empty;
                    TeaserMini = String.Empty;
                    UrlToApi = String.Empty;
                    UrlToHtml = String.Empty;

                    MessageBox.Show("That's weird.  We didn't get anything back from NPR.  Any chance you just lost your internet connection?");
                }
                else
                {
                    Title = fromValues.Title;
                    Teaser = fromValues.Teaser;
                    TeaserMini = fromValues.TeaserMini;
                    UrlToApi = fromValues.UrlToApi;
                    UrlToHtml = fromValues.UrlToHtml;

                    IStoryViewModel toValue = null;

                    foreach (var fromValue in fromValues.Stories)
                    {
                        toValue = ObjectFactoryInstance.Create<IStoryViewModel>();

                        toValue.Load(fromValue);

                        this.Stories.Add(toValue);
                    }
                }    
            }
            finally
            {
                IsProgressBarVisible = false;   
            }
        }

        private const string IdPropertyName = "Id";

        private string m_Id;
        public string Id
        {
            get
            {
                return m_Id;
            }
            set
            {
                m_Id = value;
                RaisePropertyChanged(IdPropertyName);
            }
        }

        private const string TitlePropertyName = "Title";

        private string m_Title;
        public string Title
        {
            get
            {
                return m_Title;
            }
            set
            {
                m_Title = value;
                RaisePropertyChanged(TitlePropertyName);
            }
        }

        private ObservableCollection<IStoryViewModel> m_Stories;
        public ObservableCollection<IStoryViewModel> Stories
        {
            get { return m_Stories; }
        }

        private const string TeaserPropertyName = "Teaser";

        private string m_Teaser;
        public string Teaser
        {
            get
            {
                return m_Teaser;
            }
            set
            {
                m_Teaser = value;
                RaisePropertyChanged(TeaserPropertyName);
            }
        }

        private const string TeaserMiniPropertyName = "TeaserMini";

        private string m_TeaserMini;
        public string TeaserMini
        {
            get
            {
                return m_TeaserMini;
            }
            set
            {
                m_TeaserMini = value;
                RaisePropertyChanged(TeaserMiniPropertyName);
            }
        }

        private const string UrlToHtmlPropertyName = "UrlToHtml";

        private string m_UrlToHtml;
        public string UrlToHtml
        {
            get
            {
                return m_UrlToHtml;
            }
            set
            {
                m_UrlToHtml = value;
                RaisePropertyChanged(UrlToHtmlPropertyName);
            }
        }

        private const string UrlToApiPropertyName = "UrlToApi";

        private string m_UrlToApi;
        public string UrlToApi
        {
            get
            {
                return m_UrlToApi;
            }
            set
            {
                m_UrlToApi = value;
                RaisePropertyChanged(UrlToApiPropertyName);
            }
        }

        private const string SelectedItemPropertyName = "SelectedItem";

        private IStoryViewModel m_SelectedItem;
        public IStoryViewModel SelectedItem
        {
            get
            {
                return m_SelectedItem;
            }
            set
            {
                if (m_SelectedItem != value)
                {
                    m_SelectedItem = value;

                    ViewModelLocator.Instance.CurrentStory = m_SelectedItem;

                    RaisePropertyChanged(SelectedItemPropertyName);

                    if (OnSelectionChanged != null)
                    {
                        OnSelectionChanged(this, new EventArgs());
                    }
                }
            }
        }

        public event EventHandler OnSelectionChanged;

        private const string IsProgressBarVisiblePropertyName = "IsProgressBarVisible";

        private bool m_IsProgressBarVisible;
        public bool IsProgressBarVisible
        {
            get
            {
                return m_IsProgressBarVisible;
            }
            set
            {
                m_IsProgressBarVisible = value;
                RaisePropertyChanged(IsProgressBarVisiblePropertyName);
            }
        }

        private const string ProgressBarMessagePropertyName = "ProgressBarMessage";

        private string m_ProgressBarMessage;
        public string ProgressBarMessage
        {
            get
            {
                return m_ProgressBarMessage;
            }
            set
            {
                m_ProgressBarMessage = value;
                RaisePropertyChanged(ProgressBarMessagePropertyName);
            }
        }
    }
}
