using System;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.Common.Models;
using Benday.Npr.Phone.Common;
using System.Net;
using System.IO;
using System.Linq;

namespace Benday.Npr.Phone.DataAccess
{
    public class StoryRepository : 
        RepositoryBase, IStoryRepository
    {
        public event EventHandler<StoryListRefreshedEventArgs> StoryListRefreshed;

        public StoryRepositoryTestModeConfiguration TestModeConfiguration { get; set; }

        public StoryRepository(
            IObjectFactory factory, 
            IStoryXmlAdapter adapter)
            : base(factory)
        {
            if (adapter == null)
                throw new ArgumentNullException("adapter", "adapter is null.");

            m_AdapterInstance = adapter;
        }

        private IStoryXmlAdapter m_AdapterInstance;

        public IStoryList GetStories(string programId)
        {
            var returnValue = ObjectFactoryInstance.Create<IStoryList>();

            var fileContents = DataAccessUtility.ReadFile(
                DataAccessUtility.GetProgramStoriesFilename(programId));

            if (String.IsNullOrEmpty(fileContents) == false)
            {
                // turn the cached values into objects and return
                returnValue = ProcessStories(programId, fileContents);
            }
            else
            {
                // no cached stories for this program id
                RefreshStories(programId);
            }

            return returnValue;
        }

        public void RefreshStories(string programId)
        {
            if (String.IsNullOrEmpty(programId))
                throw new ArgumentException("programId is null or empty.", "programId");

            var action = new RefreshStoriesReturnResultAction(
                new ReturnResult<IStoryList>(), programId);

            if (programId.StartsWith("RSS_FEED.") == true)
            {
                // RSS_FEED.ITS_ALL_POLITICS
                // RSS_FEED.NPR_HOURLY_NEWS
                GetStoriesForRssFeed(action, programId);
            }
            else
            {
                GetStoriesForProgram(action, programId);
            }
        }

        private void GetStoriesForRssFeed(RefreshStoriesReturnResultAction action, string id)
        {
            string url;

            if (id == "RSS_FEED.NPR_HOURLY_NEWS")
            {
                url = "http://www.npr.org/rss/podcast.php?id=500005";
            }
            else if (id == "RSS_FEED.ITS_ALL_POLITICS")
            {
                url = "http://www.npr.org/rss/podcast.php?id=510068";
            }
            else if (id == "RSS_FEED.CAR_TALK")
            {
                url = "http://www.npr.org/rss/podcast.php?id=510208";
            }
            else
            {
                throw new InvalidOperationException(String.Format("Unknown rss feed id '{0}'.", id));
            }

            HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;

            request.BeginGetResponse(ProcessStoriesFromRssFeedResult,
                new RefreshStoriesReturnResult(request, action));
        }

        private void GetStoriesForProgram(RefreshStoriesReturnResultAction action, string id)
        {
            if (TestModeConfiguration != null)
            {
                ProcessStories(action, TestModeConfiguration.TestData);                
            }
            else
            {
                string url = String.Format(
                    "http://api.npr.org/query?id={0}&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001", 
                    id);

                HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;

                request.BeginGetResponse(ProcessStoriesFromQueryResult,
                    new RefreshStoriesReturnResult(request, action));
            }
        }

        private void WriteResponseToDisk(
            string programId, 
            string writeThisToDisk)
        {
            var filename = 
                DataAccessUtility.GetProgramStoriesFilename(
                programId);

            DataAccessUtility.SaveFile(filename, writeThisToDisk);
        }

        private IStoryList ProcessStories(string programId, string xml)
        {
            if (programId.StartsWith("RSS_FEED.") == true)
            {
                // RSS_FEED.ITS_ALL_POLITICS
                // RSS_FEED.NPR_HOURLY_NEWS
                var adapter = ObjectFactoryInstance.Create<IRssFeedXmlAdapter>();

                var returnValue = ObjectFactoryInstance.Create<IStoryList>();

                adapter.Adapt(xml, returnValue);

                WriteResponseToDisk(programId, xml);

                return returnValue;
            }
            else
            {
                var adapter = ObjectFactoryInstance.Create<IStoryXmlAdapter>();

                var returnValue = ObjectFactoryInstance.Create<IStoryList>();

                adapter.Adapt(xml, returnValue);

                WriteResponseToDisk(programId, xml);

                return returnValue;
            }            
        }

        private void ProcessStories(
            RefreshStoriesReturnResultAction action, string xml)
        {
            IStoryList returnValue = ProcessStories(action.ProgramId, xml);

            System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                () => action.Action.Notify(returnValue));

            System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                () => OnStoryListRefreshed(
                    new StoryListRefreshedEventArgs(action.ProgramId)));
        }

        private void ProcessStoriesFromQueryResult(IAsyncResult asyncResult)
        {
            var action = asyncResult.AsyncState as RefreshStoriesReturnResult;

            if (action == null)
            {
                throw new InvalidOperationException("User state was the wrong type.");
            }

            try
            {
                var response = action.Request.EndGetResponse(asyncResult);

                using (var stream = new StreamReader(response.GetResponseStream()))
                {
                    string xml = stream.ReadToEnd();

                    ProcessStories(action.ReturnResult, xml);
                }
            }
            catch (Exception)
            {
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Action.Notify(null as IStoryList));
            }
        }

        private void ProcessStoriesFromRssFeedResult(IAsyncResult asyncResult)
        {
            var action = asyncResult.AsyncState as RefreshStoriesReturnResult;

            if (action == null)
            {
                throw new InvalidOperationException("User state was the wrong type.");
            }

            try
            {
                var response = action.Request.EndGetResponse(asyncResult);

                using (var stream = new StreamReader(response.GetResponseStream()))
                {
                    string xml = stream.ReadToEnd();

                    var adapter = ObjectFactoryInstance.Create<IRssFeedXmlAdapter>();

                    var returnValue = ObjectFactoryInstance.Create<IStoryList>();

                    adapter.Adapt(xml, returnValue);

                    WriteResponseToDisk(action.ReturnResult.ProgramId, xml);

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Action.Notify(returnValue));

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => OnStoryListRefreshed(
                            new StoryListRefreshedEventArgs(action.ReturnResult.ProgramId)));
                }
            }
            catch (Exception)
            {
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Action.Notify(null as IStoryList));
            }
        }

        private IStory GetStoryNotFoundStory(string storyId)
        {
            var story = ObjectFactoryInstance.Create<IStory>();

            story.Id = storyId;
            story.Title = "Story not found";

            return story;
        }

        public IStory GetStory(
            string programId, string storyId)
        {
            var stories = GetStories(programId);

            if (stories == null || stories.Stories == null || stories.Stories.Count == 0)
            {
                return GetStoryNotFoundStory(storyId);
            }
            else
            {
                IStory matchingStory = null;
                
                if (storyId == "RSS_FEED.NPR_HOURLY_NEWS")
                {
                    matchingStory = (from temp in stories.Stories
                                     select temp).FirstOrDefault();    
                }
                else
                {
                    matchingStory = (from temp in stories.Stories
                                     where temp.Id == storyId
                                     select temp).FirstOrDefault();    
                }                

                if (matchingStory == null)
                {
                    return GetStoryNotFoundStory(storyId);
                }
                else
                {
                    return matchingStory;    
                }                
            }
        }

        public void SaveCurrentStoryId(string currentId)
        {
            var doc = XmlUtility.StringToXDocument("<info />");

            if (currentId == null)
            {
                currentId = String.Empty;
            }
            else
            {
                currentId = currentId.Trim();
            }

            XmlUtility.SetChildElement(doc.Root, "CurrentStoryId", currentId);

            DataAccessUtility.SaveFile(DataAccessConstants.FilenameStoryInformation, doc);
        }

        public string GetCurrentStoryId()
        {
            var contents = DataAccessUtility.ReadFile(DataAccessConstants.FilenameStoryInformation);

            if (contents == null)
            {
                return null;
            }
            else
            {
                var doc = XmlUtility.StringToXDocument(contents);

                string returnValue =
                    XmlUtility.GetChildElementValue(doc.Root, "CurrentStoryId", null);

                if (returnValue == null || returnValue == String.Empty)
                {
                    return null;
                }
                else
                {
                    return returnValue;
                }
            }
        }

        public virtual void OnStoryListRefreshed(StoryListRefreshedEventArgs ea)
        {
            if (StoryListRefreshed != null)
                StoryListRefreshed(this, ea);
        }

        private void ProcessMp3ResolveResult(IAsyncResult asyncResult)
        {
            var action = asyncResult.AsyncState as WebRequestReturnResult<string>;

            if (action == null)
            {
                throw new InvalidOperationException("User state was the wrong type.");
            }

            try
            {
                var response = action.Request.EndGetResponse(asyncResult);

                using (var stream = new StreamReader(response.GetResponseStream()))
                {
                    string m3uPlaylistContents = stream.ReadToEnd();

                    StringReader playlistReader = new StringReader(m3uPlaylistContents);

                    string urlToMp3File = playlistReader.ReadLine();

                    while (urlToMp3File != null && urlToMp3File.StartsWith("http://") == false)
                    {
                        urlToMp3File = playlistReader.ReadLine();
                    }

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(urlToMp3File));
                }
            }
            catch (Exception ex)
            {
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(
                        () => action.ReturnResult.Notify(ex));
            }
        }

        public void ResolveMp3UrlFromM3u(ReturnResult<string> action, string m3uUri)
        {
            if (System.String.IsNullOrEmpty(m3uUri))
                throw new ArgumentNullException("m3uUri", "m3uUri is null.");
            if (action == null)
                throw new ArgumentNullException("action", "action is null.");

            try
            {
                HttpWebRequest request = WebRequest.Create(m3uUri) as HttpWebRequest;

                request.BeginGetResponse(ProcessMp3ResolveResult, new WebRequestReturnResult<string>(request, action));
            }
            catch (Exception)
            {
                throw;
            }
        }
    }
}