using System;
using System.Linq;
using System.Xml.Linq;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.Common;
using Benday.Npr.Phone.Common.Models;
using Benday.Npr.Phone.DataAccess;

namespace Benday.Npr.DataAccess
{
    public class StoryXmlAdapter : XmlAdapterBase, IStoryXmlAdapter
    {
        public StoryXmlAdapter(IObjectFactory factory)
        {
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        private IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }

        public void Adapt(string fromValue, IStoryList toValue)
        {
            if (String.IsNullOrEmpty(fromValue))
                throw new ArgumentException("fromValue is null or empty.", "fromValue");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            var doc = StringToXDocument(fromValue);

            Adapt(doc.Root, toValue);
        }

        private void Adapt(XElement fromValue, IStoryList toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.UrlToHtml = XmlUtility.GetChildElementValue(fromValue, "link", "type", "html", String.Empty);
            toValue.UrlToApi = XmlUtility.GetChildElementValue(fromValue, "link", "type", "api", String.Empty);
            toValue.Title = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty));
            toValue.Teaser = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "teaser", String.Empty));
            toValue.TeaserMini = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "miniTeaser", String.Empty));
            
            var fromStories = fromValue.Descendants("story");

            IStory toStory;

            if (fromStories != null)
            {
                foreach (var fromStory in fromStories)
                {
                    toStory = ObjectFactoryInstance.Create<IStory>();

                    Adapt(fromStory, toStory);

                    toValue.Stories.Add(toStory);
                }
            }
        }

        public void Adapt(string fromValue, IStory toValue)
        {
            if (String.IsNullOrEmpty(fromValue))
                throw new ArgumentException("fromValue is null or empty.", "fromValue");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            var doc = StringToXDocument(fromValue);

            Adapt(doc.Root, toValue);
        }

        private void Adapt(XElement fromValue, IStory toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = XmlUtility.GetAttributeValue(fromValue, "id", String.Empty);
            toValue.UrlToHtml = XmlUtility.GetChildElementValue(fromValue, "link", "type", "html", String.Empty);
            toValue.UrlToApi = XmlUtility.GetChildElementValue(fromValue, "link", "type", "api", String.Empty);
            toValue.UrlShort = XmlUtility.GetChildElementValue(fromValue, "link", "type", "short", String.Empty);
            toValue.Title = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "title", String.Empty));
            toValue.Teaser = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "teaser", String.Empty));
            toValue.TeaserMini = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "miniTeaser", String.Empty));
            toValue.Slug = DataAccessUtility.ReplaceUndisplayableHtml(XmlUtility.GetChildElementValue(fromValue, "slug", String.Empty));
            toValue.StoryDate = XmlUtility.GetChildElementValue(fromValue, "storyDate", DateTime.MinValue);
            toValue.PublishDate = XmlUtility.GetChildElementValue(fromValue, "pubDate", DateTime.MinValue);

            AdaptShowInformation(fromValue, toValue);
            AdaptOrganizationInformation(fromValue, toValue);
            AdaptAudioInformation(fromValue, toValue);

            AdaptParagraphs(fromValue, toValue);
            AdaptByline(fromValue, toValue);
        }

        private void AdaptByline(XElement fromValue, IStory toValue)
        {
            var element = XmlUtility.GetChildElement(fromValue, "byline");

            if (element == null)
            {
                toValue.Author = String.Empty;
            }
            else
            {
                toValue.Author = XmlUtility.GetChildElementValue(element, "name", String.Empty);
            }            
        }

        private void AdaptParagraphs(XElement fromValue, IStory toValue)
        {
            var element = XmlUtility.GetChildElement(fromValue, "text");

            if (element == null)
            {
                // there isn't a text element and therefore no paragraphs
                // Create a paragraph from the the teaser

                var toParagraph = ObjectFactoryInstance.Create<IParagraph>();
                toParagraph.Text = toValue.Teaser;

                toValue.Paragraphs.Add(toParagraph);
            }
            else
            {
                var paragraphs = element.Descendants("paragraph");

                IParagraph toParagraph;

                foreach (var fromParagraph in paragraphs)
                {
                    toParagraph = ObjectFactoryInstance.Create<IParagraph>();

                    toParagraph.Text = DataAccessUtility.ReplaceUndisplayableHtml(fromParagraph.Value);

                    toValue.Paragraphs.Add(toParagraph);
                }
            }
        }

        private void AdaptAudioInformation(XElement fromValue, IStory toValue)
        {
            var audioElement = XmlUtility.GetChildElement(fromValue, "audio");

            if (audioElement == null)
            {
                toValue.Duration = 0;
                toValue.UrlToMp3 = String.Empty;
                toValue.UrlToWindowsMedia = String.Empty;
            }
            else
            {
                toValue.Duration = XmlUtility.GetChildElementValue(audioElement, "duration", 0);

                var formatElement = XmlUtility.GetChildElement(audioElement, "format");

                if (formatElement != null)
                {
                    toValue.UrlToMp3 = XmlUtility.GetChildElementValue(formatElement, "mp3", String.Empty);
                    toValue.UrlToWindowsMedia =
                        XmlUtility.GetChildElementValue(formatElement, "wm", String.Empty);    
                }
            }
        }

        private void AdaptOrganizationInformation(XElement fromValue, IStory toValue)
        {
            var element = XmlUtility.GetChildElement(fromValue, "organization");

            if (element == null)
            {
                toValue.OrganizationName = String.Empty;
                toValue.OrganizationUrl = String.Empty;
                toValue.OrganizationAbbreviation = String.Empty;
            }
            else
            {
                toValue.OrganizationName = XmlUtility.GetChildElementValue(element, "name", String.Empty);
                toValue.OrganizationUrl = XmlUtility.GetChildElementValue(element, "website", String.Empty);
                toValue.OrganizationAbbreviation =
                    XmlUtility.GetAttributeValue(element, "orgAbbr", String.Empty);
            }
        }

        private void AdaptShowInformation(XElement fromValue, IStory toValue)
        {
            var element = XmlUtility.GetChildElement(fromValue, "show");
    
            if (element == null)
            {
                toValue.ShowDate = DateTime.MinValue;
                toValue.ShowName = String.Empty;
            }
            else
            {
                toValue.ShowDate = XmlUtility.GetChildElementValue(element, "showDate", DateTime.Now);
                toValue.ShowName = XmlUtility.GetChildElementValue(element, "program", String.Empty);   
            }
        }
    }
}
