using System;
using System.Collections.Generic;
using Benday.Npr.Phone.Presentation.Interfaces;
using Benday.Npr.Phone.Common.Models;
using Benday.Npr.Phone.Common;
using Benday.Npr.Phone.Common.Repositories;

namespace Benday.Npr.Phone.Presentation
{
    public class StoryListViewModel : ObjectFactoryViewModel, IStoryListViewModel
    {
        public StoryListViewModel(
            IProgramRepository programRepositoryInstance,
            IStoryRepository storyRepositoryInstance,
            IObjectFactory factory)
            : base(factory)
        {
            if (programRepositoryInstance == null)
                throw new ArgumentNullException("programRepositoryInstance", "programRepositoryInstance is null.");
            if (storyRepositoryInstance == null)
                throw new ArgumentNullException("storyRepositoryInstance", "storyRepositoryInstance is null.");
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_Stories = new SingleSelectListViewModel(new List<ISelectableItem>());
            m_Stories.OnItemSelected += new EventHandler(m_Stories_OnItemSelected);

            m_StoryRepositoryInstance = storyRepositoryInstance;
            m_StoryRepositoryInstance.StoryListRefreshed += new EventHandler<StoryListRefreshedEventArgs>(m_StoryRepositoryInstance_StoryListRefreshed);

            m_ProgramRepositoryInstance = programRepositoryInstance;
        }

        void m_StoryRepositoryInstance_StoryListRefreshed(object sender, StoryListRefreshedEventArgs e)
        {
            Load();
        }

        void m_Stories_OnItemSelected(object sender, EventArgs e)
        {
            HandleStorySelected();
        }

        private void HandleStorySelected()
        {
            if (Stories.SelectedItem == null)
            {
                CurrentStoryId = null;
                StoryRepositoryInstance.SaveCurrentStoryId(null);
            }
            else
            {
                CurrentStoryId = Stories.SelectedItem.Id;
                StoryRepositoryInstance.SaveCurrentStoryId(CurrentStoryId);
            }
        }

        private IProgramRepository m_ProgramRepositoryInstance;
        private IProgramRepository ProgramRepositoryInstance
        {
            get { return m_ProgramRepositoryInstance; }
        }

        private IStoryRepository m_StoryRepositoryInstance;
        private IStoryRepository StoryRepositoryInstance
        {
            get { return m_StoryRepositoryInstance; }
        }

        public void Load()
        {
            CurrentStoryId = StoryRepositoryInstance.GetCurrentStoryId();

            PopulateProgramInfo();
            IsProgressBarVisible = true;
            ProgressBarMessage = "Loading stories...";

            Id = CurrentProgramId;

            var stories = StoryRepositoryInstance.GetStories(CurrentProgramId);

            Populate(stories);
        }

        public void Refresh()
        {
            StoryRepositoryInstance.RefreshStories(CurrentProgramId);
        }

        private void PopulateProgramInfo()
        {
            string programId = ProgramRepositoryInstance.GetCurrentProgramId();
            
            if (programId != null)
            {
                var program = ProgramRepositoryInstance.GetById(programId);

                if (program == null)
                {
                    CurrentProgramId = null;
                    CurrentProgramTitle = null;
                }
                else
                {
                    CurrentProgramId = programId;
                    CurrentProgramTitle = program.Name;
                }
            }
        }

        private void Populate(IStoryList fromValues)
        {
            ProgressBarMessage = "Parsing stories...";

            try
            {
                this.Stories.Clear();

                if (fromValues == null)
                {
                    Title = String.Empty;
                    Teaser = String.Empty;
                    TeaserMini = String.Empty;
                    UrlToApi = String.Empty;
                    UrlToHtml = String.Empty;

                    RequestMessageBox(
                        "That's weird.  We didn't get anything back from NPR.  Any chance you just lost your internet connection?");
                }
                else
                {
                    Title = fromValues.Title;
                    Teaser = fromValues.Teaser;
                    TeaserMini = fromValues.TeaserMini;
                    UrlToApi = fromValues.UrlToApi;
                    UrlToHtml = fromValues.UrlToHtml;

                    SelectableItem toValue = null;

                    foreach (var fromValue in fromValues.Stories)
                    {
                        toValue = new StorySelectableItem(
                            false, 
                            fromValue.Id, 
                            fromValue.Title, 
                            fromValue.Id, 
                            fromValue.Teaser, 
                            fromValue.HasUrlToMp3, 
                            fromValue.PublishDate);

                        this.Stories.Add(toValue);
                    }
                }
            }
            finally
            {
                IsProgressBarVisible = false;
            }
        }

        private const string IdPropertyName = "Id";

        private string m_Id;
        public string Id
        {
            get
            {
                return m_Id;
            }
            set
            {
                m_Id = value;
                RaisePropertyChanged(IdPropertyName);
            }
        }

        private const string CurrentStoryIdPropertyName = "CurrentStoryId";

        private string m_CurrentStoryId;
        public string CurrentStoryId
        {
            get
            {
                return m_CurrentStoryId;
            }
            set
            {
                m_CurrentStoryId = value;
                RaisePropertyChanged(CurrentStoryIdPropertyName);
            }
        }

        private const string CurrentProgramIdPropertyName = "CurrentProgramId";

        private string m_CurrentProgramId;
        public string CurrentProgramId
        {
            get
            {
                return m_CurrentProgramId;
            }
            set
            {
                m_CurrentProgramId = value;
                RaisePropertyChanged(CurrentProgramIdPropertyName);
            }
        }

        private const string CurrentProgramTitlePropertyName = "CurrentProgramTitle";

        private string m_CurrentProgramTitle;
        public string CurrentProgramTitle
        {
            get
            {
                return m_CurrentProgramTitle;
            }
            set
            {
                m_CurrentProgramTitle = value;
                RaisePropertyChanged(CurrentProgramTitlePropertyName);
            }
        }

        private const string TitlePropertyName = "Title";

        private string m_Title;
        public string Title
        {
            get
            {
                return m_Title;
            }
            set
            {
                m_Title = value;
                RaisePropertyChanged(TitlePropertyName);
            }
        }

        private SingleSelectListViewModel m_Stories;
        public SingleSelectListViewModel Stories
        {
            get { return m_Stories; }
        }

        private const string TeaserPropertyName = "Teaser";

        private string m_Teaser;
        public string Teaser
        {
            get
            {
                return m_Teaser;
            }
            set
            {
                m_Teaser = value;
                RaisePropertyChanged(TeaserPropertyName);
            }
        }

        private const string TeaserMiniPropertyName = "TeaserMini";

        private string m_TeaserMini;
        public string TeaserMini
        {
            get
            {
                return m_TeaserMini;
            }
            set
            {
                m_TeaserMini = value;
                RaisePropertyChanged(TeaserMiniPropertyName);
            }
        }

        private const string UrlToHtmlPropertyName = "UrlToHtml";

        private string m_UrlToHtml;
        public string UrlToHtml
        {
            get
            {
                return m_UrlToHtml;
            }
            set
            {
                m_UrlToHtml = value;
                RaisePropertyChanged(UrlToHtmlPropertyName);
            }
        }

        private const string UrlToApiPropertyName = "UrlToApi";

        private string m_UrlToApi;
        public string UrlToApi
        {
            get
            {
                return m_UrlToApi;
            }
            set
            {
                m_UrlToApi = value;
                RaisePropertyChanged(UrlToApiPropertyName);
            }
        }

        private const string IsProgressBarVisiblePropertyName = "IsProgressBarVisible";

        private bool m_IsProgressBarVisible;
        public bool IsProgressBarVisible
        {
            get
            {
                return m_IsProgressBarVisible;
            }
            set
            {
                m_IsProgressBarVisible = value;
                RaisePropertyChanged(IsProgressBarVisiblePropertyName);
            }
        }

        private const string ProgressBarMessagePropertyName = "ProgressBarMessage";

        private string m_ProgressBarMessage;
        public string ProgressBarMessage
        {
            get
            {
                return m_ProgressBarMessage;
            }
            set
            {
                m_ProgressBarMessage = value;
                RaisePropertyChanged(ProgressBarMessagePropertyName);
            }
        }
    }
}
