using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Microsoft.Silverlight.Testing;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.DataAccess;
using Benday.Npr.Phone.Common;
using System.Collections.Generic;
using Benday.Npr.Phone.Common.Models;

namespace Benday.Npr.Phone.UnitTests.DataAccess
{
    [TestClass]
    [Tag("Repository")]
    public class ProgramRepositoryFixture : UnitTestBase
    {
        private bool m_WasNotifyCalled = false;
        private IList<IProgramInfo> m_DataReturnedFromGetAllCall = null;
        private IProgramInfo m_DataReturnedFromGetByIdCall = null;

        protected override void OnTestInitialize()
        {
            UnitTestUtility.WipeProgramDataInIsolatedStorage();

            m_RepositoryInstance = null;
            m_WasNotifyCalled = false;
            m_DataReturnedFromGetAllCall = null;
            m_DataReturnedFromGetByIdCall = null;
        }

        private ProgramRepository m_RepositoryInstance;
        public ProgramRepository RepositoryInstance
        {
            get
            {
                if (m_RepositoryInstance == null)
                {
                    m_RepositoryInstance = 
                        ObjectFactoryInstance.Create<IProgramRepository>() as ProgramRepository;
                }
                
                return m_RepositoryInstance;
            }
        }

        [TestMethod]
        public void GetAnInstanceOfIProgramRepositoryReturnsNonNullAndTypeofProgramRepository()
        {
            var instance = ObjectFactoryInstance.Create<IProgramRepository>();

            Assert.IsNotNull(instance, "Instance of IProgramRepository returned from ObjectFactory was null.");
            Assert.IsInstanceOfType(instance, typeof(ProgramRepository), "Got the wrong type back.");
        }

        [TestMethod]
        public void RepositoryInstancePropertyOnTestInitializesToNonNull()
        {
            Assert.IsNotNull(RepositoryInstance, "Instance was null.");
        }
        
        [TestMethod]
        public void GetAll()
        {
            RepositoryInstance.GetAll(new ReturnResult<IList<IProgramInfo>>(GetAllHandler));

            Assert.IsTrue(m_WasNotifyCalled, "Notify() wasn't called.");

            Assert.AreNotEqual<int>(0, m_DataReturnedFromGetAllCall.Count, "Result count was 0.");
        }

        private void GetAllHandler(ReturnResult<IList<IProgramInfo>> result)
        {
            m_WasNotifyCalled = true;
            m_DataReturnedFromGetAllCall = result.Result;

            Assert.IsNull(result.Error, "Got an exception.  '{0}'", result.Error);
            Assert.IsNotNull(result.Result, "Did not get a result.");
        }

        [TestMethod]
        public void GetById()
        {
            string expectedId = "RSS_FEED.ITS_ALL_POLITICS";
            RepositoryInstance.GetById(new ReturnResult<IProgramInfo>(GetByIdHandler), expectedId);

            Assert.IsTrue(m_WasNotifyCalled, "Notify() wasn't called.");

            Assert.IsNotNull(m_DataReturnedFromGetByIdCall, "Got null data from call.");

            Assert.AreEqual<string>(expectedId, m_DataReturnedFromGetByIdCall.Id, "Id value was wrong.");
            Assert.AreEqual<string>("It's All Politics", m_DataReturnedFromGetByIdCall.Name, "Name was wrong.");
        }

        private void GetByIdHandler(ReturnResult<IProgramInfo> result)
        {
            m_WasNotifyCalled = true;
            m_DataReturnedFromGetByIdCall = result.Result;

            Assert.IsNull(result.Error, "Got an exception.  '{0}'", result.Error);
            Assert.IsNotNull(result.Result, "Did not get a result.");
        }

        [TestMethod]
        public void AssertProgramInforationFileDoesNotExistInStorageAtStartOfTest()
        {
            Assert.IsFalse(
                UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation),
                "Program information file should not exist.");
        }

        [TestMethod]
        public void SaveCurrentProgramId()
        {
            RepositoryInstance.SaveCurrentProgramId("asdf");

            Assert.IsTrue(
                UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation),
                "Program information file does not exist.");
        }

        [TestMethod]
        public void SaveCurrentProgramIdAcceptsNull()
        {
            RepositoryInstance.SaveCurrentProgramId(null);

            Assert.IsTrue(
                UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation),
                "Program information file does not exist.");
        }

        [TestMethod]
        public void GetCurrentProgramId()
        {
            string expected = "asdf";
            
            RepositoryInstance.SaveCurrentProgramId(expected);

            var actual = RepositoryInstance.GetCurrentProgramId();

            Assert.AreEqual<string>(expected, actual, "CurrentProgramId did not match.");
        }

        [TestMethod]
        public void GetCurrentProgramIdWhenValueIsEmptyStringReturnsNull()
        {
            string expected = null;

            RepositoryInstance.SaveCurrentProgramId(String.Empty);

            var actual = RepositoryInstance.GetCurrentProgramId();

            Assert.AreEqual<string>(expected, actual, "CurrentProgramId did not match.");
        }

        [TestMethod]
        public void GetCurrentProgramIdWhenValueIsNullStringReturnsNull()
        {
            string expected = null;

            RepositoryInstance.SaveCurrentProgramId(null);

            var actual = RepositoryInstance.GetCurrentProgramId();

            Assert.AreEqual<string>(expected, actual, "CurrentProgramId did not match.");
        }
    }
}
