using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Phone.DataAccess;
using Benday.Npr.Phone.Common.Repositories;
using System.Threading;
using Microsoft.Silverlight.Testing;
using Benday.Npr.Phone.Common.Models;

namespace Benday.Npr.Phone.UnitTests.DataAccess
{
    [TestClass]
    public class StoryRepositoryFixture : UnitTestBase
    {
        public StoryRepositoryFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            UnitTestUtility.WipeProgramDataInIsolatedStorage();

            m_RepositoryInstance = null;
            m_ProgramRepositoryInstance = null;
        }

        private StoryRepository m_RepositoryInstance;
        public StoryRepository RepositoryInstance
        {
            get
            {
                if (m_RepositoryInstance == null)
                {
                    m_RepositoryInstance =
                        ObjectFactoryInstance.Create<IStoryRepository>() as StoryRepository;

                    m_RepositoryInstance.TestModeConfiguration =
                        new StoryRepositoryTestModeConfiguration() 
                        { TestData = UnitTestResources.AllThingsConsideredStories };
                }

                return m_RepositoryInstance;
            }
        }

        private ProgramRepository m_ProgramRepositoryInstance;
        public ProgramRepository ProgramRepositoryInstance
        {
            get
            {
                if (m_ProgramRepositoryInstance == null)
                {
                    m_ProgramRepositoryInstance =
                        ObjectFactoryInstance.Create<IProgramRepository>() as ProgramRepository;
                }

                return m_ProgramRepositoryInstance;
            }
        }

        [TestMethod]
        public void GetProgramStoriesFilename()
        {
            string expected = "ProgramStoriesList_18.xml";

            string actual = DataAccessUtility.GetProgramStoriesFilename("18");

            Assert.AreEqual<string>(expected, actual, "Filename was wrong.");
        }

        [TestMethod]
        public void AssertProgramStoriesFileDoesNotExistInStorageAtStartOfTest()
        {
            Assert.IsFalse(
                UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation),
                "Program information file should not exist.");
        }

        [TestMethod]
        public void WhenNoCachedStoryListIsAvailableForTheRequestedProgramRefreshFromServer()
        {
            var actual = RepositoryInstance.GetStories("18");

            string filename = DataAccessUtility.GetProgramStoriesFilename("18");

            Assert.IsTrue(UnitTestUtility.FileExists(filename),
                "Cached story does not exist.  Refresh() was probalby not called by repository.");
        }

        [TestMethod]
        public void WhenNoCachedStoryListIsAvailableReturnAnEmptyStoryList()
        {
            var actual = RepositoryInstance.GetStories("18");

            Assert.IsNotNull(actual, "Should not be null.");
            Assert.IsNotNull(actual.Stories, "Stories collection was null.");
            Assert.AreEqual<int>(0, actual.Stories.Count, "Story count should be 0.");
        }

        [TestMethod]
        public void WhenCachedStoryListIsAvailableReturnListOfStories()
        {
            RepositoryInstance.RefreshStories("18");
            
            var actual = RepositoryInstance.GetStories("18");

            Assert.IsNotNull(actual, "Should not be null.");
            Assert.IsNotNull(actual.Stories, "Stories collection was null.");
            Assert.AreNotEqual<int>(0, actual.Stories.Count, "Story count should not be 0.");
        }

        [TestMethod]
        public void WhenStoryListIsRefreshedFromServerTheyAreStoredToDisk()
        {
            RepositoryInstance.RefreshStories("18");

            Assert.IsTrue(UnitTestUtility.FileExists(
                DataAccessUtility.GetProgramStoriesFilename("18")),
                "Cached file did not exist.");
        }

        [TestMethod]
        [Asynchronous]
        [Tag("RefreshEvent")]
        public void WhenRefreshedRepositoryRaisesRefreshEvent()
        {
            // turn off test mode...this method will actually call NPR
            RepositoryInstance.TestModeConfiguration = null;

            StoryRepositoryTester tester = new StoryRepositoryTester(RepositoryInstance);

            string programIdForAllThingsConsidered = "2";

            EnqueueCallback(() => RepositoryInstance.RefreshStories(programIdForAllThingsConsidered));

            EnqueueDelay(2000);

            EnqueueCallback(() => 
            tester.AssertEvent(programIdForAllThingsConsidered));

            EnqueueTestComplete();
        }

        [TestMethod]
        public void GetStoryByProgramIdAndStoryId()
        {
            RepositoryInstance.RefreshStories("18");

            string expectedStoryId = "131216486";
            string expectedProgramId = "18";

            var actual = RepositoryInstance.GetStory(
                expectedProgramId, expectedStoryId);

            Assert.IsNotNull(actual, "Story was null.");
            Assert.AreEqual<string>(
                "In Israel, When Is A Jew Not Jewish Enough?", 
                actual.Title, 
                "Story title was wrong.");
        }

        [TestMethod]
        public void ObjectFactoryCanCreateInstanceOfIStory()
        {
            var story = ObjectFactoryInstance.Create<IStory>();

            Assert.IsNotNull(story, "Value was null.");
        }

        [TestMethod]
        public void WhenStoryNotAvailableOnDiskReturnFileNotFoundStory()
        {
            RepositoryInstance.RefreshStories("18");

            string bogusStoryId = "bogus story id";
            string expectedProgramId = "18";

            var actual = RepositoryInstance.GetStory(
                expectedProgramId, bogusStoryId);

            Assert.IsNotNull(actual, "Story was null.");

            Assert.AreEqual<string>("Story not found", actual.Title, "Title should say story not found.");
            Assert.AreEqual<string>(bogusStoryId, actual.Id, "Story id was wrong.");
        }

        [TestMethod]
        public void SaveCurrentStoryId()
        {
            RepositoryInstance.SaveCurrentStoryId("asdf");

            Assert.IsTrue(
                UnitTestUtility.FileExists(DataAccessConstants.FilenameStoryInformation),
                "Story information file does not exist.");
        }

        [TestMethod]
        public void SaveCurrentStoryIdAcceptsNull()
        {
            RepositoryInstance.SaveCurrentStoryId(null);

            Assert.IsTrue(
                UnitTestUtility.FileExists(DataAccessConstants.FilenameStoryInformation),
                "Story information file does not exist.");
        }

        [TestMethod]
        public void GetCurrentStoryId()
        {
            string expected = "asdf";

            RepositoryInstance.SaveCurrentStoryId(expected);

            var actual = RepositoryInstance.GetCurrentStoryId();

            Assert.AreEqual<string>(expected, actual, "CurrentStoryId did not match.");
        }

        [TestMethod]
        public void GetCurrentStoryIdWhenValueIsEmptyStringReturnsNull()
        {
            string expected = null;

            RepositoryInstance.SaveCurrentStoryId(String.Empty);

            var actual = RepositoryInstance.GetCurrentStoryId();

            Assert.AreEqual<string>(expected, actual, "CurrentStoryId did not match.");
        }

        [TestMethod]
        public void GetCurrentStoryIdWhenValueIsNullStringReturnsNull()
        {
            string expected = null;

            RepositoryInstance.SaveCurrentStoryId(null);

            var actual = RepositoryInstance.GetCurrentStoryId();

            Assert.AreEqual<string>(expected, actual, "CurrentStoryId did not match.");
        }
        
    }
}