using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Xml.Linq;
using Microsoft.Silverlight.Testing;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.Common.Models;

namespace Benday.Npr.Phone.UnitTests.DataAccess
{

    [TestClass]
    public class StoryXmlAdapterFixture : UnitTestBase
    {
        public StoryXmlAdapterFixture()
        {
        }

        [TestMethod]
        public void StoriesXmlAdapter_ParseAllThingsConsideredStories()
        {
            var fromValue = TestDataUtility.AllThingsConsideredStories();

            var adapter = ObjectFactoryInstance.Create<IStoryXmlAdapter>();
            var toValue = ObjectFactoryInstance.Create<IStoryList>();

            adapter.Adapt(fromValue, toValue);

            Assert.AreEqual<string>("NPR: All Things Considered", toValue.Title, "Title was wrong.");

            Assert.AreEqual<string>(
                "Every weekday, All Things Considered hosts Robert Siegel, Michele Norris and Melissa Block present the program's trademark mix of news, interviews, commentaries, reviews, and offbeat features.", 
                toValue.Teaser, "Teaser was wrong.");
            Assert.AreEqual<string>("A trademark mix of news, interviews, commentaries, reviews, and offbeat features.", 
                toValue.TeaserMini, "TeaserMini was wrong.");

            Assert.AreEqual<string>(
                "http://www.npr.org/programs/all-things-considered/?ft=3&f=2",
                toValue.UrlToHtml, 
                "UrlToHtml");

            Assert.AreEqual<string>(
                "http://api.npr.org/query/query?id=2&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001",
                toValue.UrlToApi, 
                "UrlToApi");

            Assert.IsNotNull(toValue.Stories, "Stories was null.");
            Assert.AreNotEqual<int>(0, toValue.Stories.Count, "Story count was wrong.");

            foreach (var story in toValue.Stories)
            {
                Assert.IsNotNull(story, "Story was null.");
                Assert.AreNotEqual<string>(String.Empty, story.Title, "Title should not be empty.");
                UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(story.Title, "story.Title");

                foreach (var paragraph in story.Paragraphs)
                {
                    UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(paragraph.Text, "paragraph.Text");
                }
            }
        }
        
        [TestMethod]
        [Tag("UndisplayableHtml")]
        public void StoriesXmlAdapter_ParseTellMeMoreStories_RemoveUndisplayableHtml()
        {
            var fromValue = TestDataUtility.TellMeMoreStories();

            var adapter = ObjectFactoryInstance.Create<IStoryXmlAdapter>();
            var toValue = ObjectFactoryInstance.Create<IStoryList>();

            adapter.Adapt(fromValue, toValue);

            UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(toValue.Title, "toValue.Title");
            UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(toValue.Teaser, "toValue.Teaser");
            UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(toValue.TeaserMini, "toValue.TeaserMini");

            Assert.IsNotNull(toValue.Stories, "Stories was null.");
            Assert.AreNotEqual<int>(0, toValue.Stories.Count, "Story count was wrong.");

            foreach (var story in toValue.Stories)
            {
                Assert.IsNotNull(story, "Story was null.");
                Assert.AreNotEqual<string>(String.Empty, story.Title, "Title should not be empty.");
                UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(story.Title, "story.Title");

                foreach (var paragraph in story.Paragraphs)
                {
                    UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(paragraph.Text, "paragraph.Text");
                }
            }
        }

        [TestMethod]
        public void StoriesXmlAdapter_ParseStory()
        {
            var fromValue = TestDataUtility.IndividualStory();

            var adapter = ObjectFactoryInstance.Create<IStoryXmlAdapter>();
            var toValue = ObjectFactoryInstance.Create<IStory>();

            adapter.Adapt(fromValue, toValue);

            Assert.AreEqual<string>("131216486", toValue.Id, "Id");
            Assert.AreEqual<string>("http://www.npr.org/2010/11/10/131216486/in-israel-when-is-a-jew-not-jewish-enough?ft=3&f=2",
                toValue.UrlToHtml, 
                "UrlToHtml");
            Assert.AreEqual<string>("http://api.npr.org/query?id=131216486&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001",
                toValue.UrlToApi, 
                "UrlToApi");
            Assert.AreEqual<string>("http://n.pr/9k87A3",
                toValue.UrlShort, 
                "UrlShort");

            Assert.AreEqual<string>("In Israel, When Is A Jew Not Jewish Enough?", 
                toValue.Title, 
                "Title");
            Assert.AreEqual<string>("The question of Jewish identity is troubling the Israel Defense Forces -- and it's affecting a number of American Jews. The debate highlights the fault lines between different strands of Judaism and the growing influence of ultra-Orthodox Jews in Israel.", 
                toValue.Teaser, 
                "Teaser");
            Assert.AreEqual<string>("The debate over Jewish identity highlights the growing influence of ultra-Orthodox Jews in Israel.", 
                toValue.TeaserMini, 
                "TeaserMini");

            Assert.AreEqual<string>("Middle East", toValue.Slug, "Slug");

            Assert.AreEqual<DateTime>(new DateTime(2010, 11, 10, 15, 0, 0), toValue.StoryDate, "StoryDate");
            Assert.AreEqual<DateTime>(new DateTime(2010, 11, 10, 17, 21, 0), toValue.PublishDate, "PublishDate");

            Assert.AreEqual<string>("All Things Considered", toValue.ShowName, "ShowName");
            Assert.AreEqual<DateTime>(new DateTime(2010, 11, 10, 15, 0, 0), toValue.ShowDate, "ShowDate");

            Assert.AreEqual<string>("National Public Radio", toValue.OrganizationName, "OrganizationName");
            Assert.AreEqual<string>("NPR", toValue.OrganizationAbbreviation, "OrganizationAbbreviation");
            Assert.AreEqual<string>("http://www.npr.org/", toValue.OrganizationUrl, "OrganizationUrl");

            Assert.AreEqual<int>(283, toValue.Duration, "StoryDuration");

            Assert.AreEqual<string>("http://api.npr.org/m3u/1131216499-490434.m3u?orgId=1&topicId=1009&ft=3&f=2", 
                toValue.UrlToMp3, 
                "UrlToMp3");
            Assert.AreEqual<string>("http://www.npr.org/templates/dmg/dmg_wmref_em.php?id=131216499&type=1&mtype=WM&orgId=1&topicId=1009&ft=3&f=2", 
                toValue.UrlToWindowsMedia, 
                "UrlToWindowsMedia");

            Assert.IsNotNull(toValue.Paragraphs, "Paragraphs collection was null.");
            Assert.AreEqual<int>(28, toValue.Paragraphs.Count, "Paragraph count was wrong.");

            foreach (var paragraph in toValue.Paragraphs)
            {
                Assert.IsNotNull(paragraph, "Paragraph was null.");
                Assert.IsNotNull(paragraph.Text, "Paragraph.Text was null.");
                Assert.AreNotEqual<string>(String.Empty, paragraph.Text, "paragraph.Text should not be empty.");
                UnitTestUtility.AssertStringDoesNotContainUndisplayableHtmlFormatting(paragraph.Text, "paragraph.Text");
            }

            Assert.AreEqual<string>("Lourdes Garcia-Navarro", toValue.Author, "Author");
        }        
    }
}
