﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Phone.Presentation.Interfaces;
using Benday.Npr.Phone.Presentation;
using Benday.Npr.Phone.UnitTests.DataAccess;
using Benday.Npr.Phone.Common.Repositories;

namespace Benday.Npr.Phone.UnitTests.Presentation
{
    
    [TestClass]
    public class ProgramsViewModelFixture : UnitTestBase
    {
        protected override void OnTestInitialize()
        {
            UnitTestUtility.WipeProgramDataInIsolatedStorage();
            m_ViewModelInstance = null;
            base.OnTestInitialize();
        }

        private IProgramsViewModel m_ViewModelInstance;
        public IProgramsViewModel ViewModelInstance
        {
            get
            {
                if (m_ViewModelInstance == null)
                {
                    m_ViewModelInstance = ObjectFactoryInstance.Create<IProgramsViewModel>();
                }

                return m_ViewModelInstance;
            }
        }

        [TestMethod]
        public void AllProgramsViewModel_Load()
        {
            ViewModelInstance.Load();

            Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.");

            Assert.AreEqual<int>(16, ViewModelInstance.Programs.Items.Count, "Program count was wrong.");

            AssertProgram(ViewModelInstance, 0, "NPR Hourly News Summary", "RSS_FEED.NPR_HOURLY_NEWS");
            AssertProgram(ViewModelInstance, 1, "All Things Considered", "2");
            AssertProgram(ViewModelInstance, 2, "Fresh Air from WHYY", "13");
            AssertProgram(ViewModelInstance, 3, "It's All Politics", "RSS_FEED.ITS_ALL_POLITICS");
            AssertProgram(ViewModelInstance, 12, "Tell Me More", "46");
        }

        [TestMethod]
        public void ProgramsViewModel_SelectItem_TriggersUpdatedEvent_UpdatesSelectedItemId()
        {
            ViewModelInstance.Load();

            Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.");

            Assert.AreEqual<int>(16, ViewModelInstance.Programs.Items.Count, "Program count was wrong.");

            Assert.IsNull(ViewModelInstance.Programs.SelectedItem, "Nothing should be selected yet.");

            Assert.AreEqual<string>(null, ViewModelInstance.CurrentProgramId,
                "Current program id value was wrong before selection.");

            var expectedSelection = ViewModelInstance.Programs.Items[0];

            var tester = new ProgramsViewModelEventTester(ViewModelInstance);

            expectedSelection.IsSelected = true;

            Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.");

            Assert.AreEqual<string>(expectedSelection.Id, ViewModelInstance.CurrentProgramId,
                "Current program id value was wrong.");            
        }


        [TestMethod]
        public void ProgramsViewModel_SelectItem_SavesCurrentProgramIdToIsolatedStorage()
        {
            ViewModelInstance.Load();

            Assert.IsNotNull(ViewModelInstance.Programs, "Programs was null.");

            SelectProgramAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0);
            SelectProgramAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 1);
        }

        [TestMethod]
        public void ProgramsViewModel_WhenLoadMethodIsCalledCurrentProgramIdIsLoadedOnCall()
        {
            ViewModelInstance.Load();

            Assert.IsNull(ViewModelInstance.CurrentProgramId, "CurrentProgramId should be null.");

            SelectProgramAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0);

            var expectedCurrentProgramId = ViewModelInstance.CurrentProgramId;

            m_ViewModelInstance = null;
            
            ViewModelInstance.Load();

            Assert.AreEqual<string>(expectedCurrentProgramId, ViewModelInstance.CurrentProgramId, 
                "CurrentProgramId did not match expected value after call to Load().");
        }

        private void SelectProgramAndAssertIdIsSavedToIsolatedStorage(IProgramsViewModel viewModel, int index)
        {
            var selectThis = viewModel.Programs.Items[index];

            selectThis.IsSelected = true;

            var repository = ObjectFactoryInstance.Create<IProgramRepository>();

            var currentProgramId = repository.GetCurrentProgramId();

            Assert.AreEqual<string>(selectThis.Id, currentProgramId, "Program Id did not match what was stored.");
        }

        private void AssertProgram(
            IProgramsViewModel instance, 
            int programIndex, 
            string expectedTitle, string expectedId)
        {
            var actual = instance.Programs.Items[programIndex];

            Assert.AreEqual<string>(expectedTitle, actual.Text, "Text");
            Assert.AreEqual<string>(expectedId, 
                actual.Id, 
                "Id");
        }
    }
}
