using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Npr.Phone.Presentation.Interfaces;
using Benday.Npr.Phone.UnitTests.DataAccess;
using Benday.Npr.Phone.Common.Repositories;
using Benday.Npr.Phone.DataAccess;
using Benday.Npr.Phone.Presentation;
using Microsoft.Silverlight.Testing;

namespace Benday.Npr.Phone.UnitTests.Presentation
{
    [TestClass]
    public class StoryListViewModelFixture : UnitTestBase
    {
        protected override void OnTestInitialize()
        {
            UnitTestUtility.WipeProgramDataInIsolatedStorage();
            m_ViewModelInstance = null;
            m_RepositoryInstance = null;
            m_ProgramRepositoryInstance = null;
            base.OnTestInitialize();
        }

        private IStoryListViewModel m_ViewModelInstance;
        public IStoryListViewModel ViewModelInstance
        {
            get
            {
                if (m_ViewModelInstance == null)
                {
                    m_ViewModelInstance = new StoryListViewModel(
                        ProgramRepositoryInstance, 
                        RepositoryInstance, 
                        ObjectFactoryInstance);
                }

                return m_ViewModelInstance;
            }
        }

        private StoryRepository m_RepositoryInstance;
        public StoryRepository RepositoryInstance
        {
            get
            {
                if (m_RepositoryInstance == null)
                {
                    m_RepositoryInstance =
                        ObjectFactoryInstance.Create<IStoryRepository>() as StoryRepository;

                    m_RepositoryInstance.TestModeConfiguration =
                        new StoryRepositoryTestModeConfiguration() 
                        { TestData = UnitTestResources.AllThingsConsideredStories };
                }

                return m_RepositoryInstance;
            }
        }

        private IProgramRepository m_ProgramRepositoryInstance;
        public IProgramRepository ProgramRepositoryInstance
        {
            get
            {
                if (m_ProgramRepositoryInstance == null)
                {
                    m_ProgramRepositoryInstance =
                        new ProgramRepositoryFake();
                }

                return m_ProgramRepositoryInstance;
            }
        }

        [TestMethod]
        public void CreateInstanceOfStoryListViewModelViaObjectFactory()
        {
            var item = ObjectFactoryInstance.Create<IStoryListViewModel>();

            Assert.IsNotNull(item);
        }

        [TestMethod]
        public void LoadFromCache()
        {
            // force the repository to write the test data to cache
            RepositoryInstance.RefreshStories("2");

            ViewModelInstance.Load();

            UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId");
            UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramTitle, "CurrentProgramId");

            Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.");

            Assert.AreEqual<int>(10, ViewModelInstance.Stories.Items.Count, "Story count was wrong.");

            AssertStory(ViewModelInstance, 0, "In Israel, When Is A Jew Not Jewish Enough?", "131216486");
            AssertStory(ViewModelInstance, 1, "Economic Woes Spur Interest In Refinancing Plan", "131223118");
            AssertStory(ViewModelInstance, 9, "Deficit Panel Pushes Cuts To Social Security", "131223274");
        }

        [TestMethod]
        public void LoadForProgramWithoutLocalCache()
        {
            ViewModelInstance.Load();

            UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId");
            UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramTitle, "CurrentProgramId");

            Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.");

            Assert.AreEqual<int>(0, ViewModelInstance.Stories.Items.Count, "Program count was wrong.");
        }

        [TestMethod]
        [Asynchronous]
        public void LoadForProgramWithoutLocalCacheCallsRefresh()
        {
            // turn off test mode on the repository...this method will actually call NPR
            RepositoryInstance.TestModeConfiguration = null;

            // calling this should cause the load to call refresh(); 
            EnqueueCallback(() => ViewModelInstance.Load());

            EnqueueCallback(() => 
            Assert.AreEqual<int>(0, ViewModelInstance.Stories.Items.Count, "Story count at start of test was wrong."));
                        
            EnqueueDelay(3000);

            EnqueueCallback(() =>
            Assert.AreEqual<int>(10, ViewModelInstance.Stories.Items.Count, "Story collection should be populated."));

            EnqueueTestComplete();
        }

        [TestMethod]
        [Asynchronous]
        public void RefreshEventOnRepositoryPopulatesStories()
        {
            // turn off test mode on the repository...this method will actually call NPR
            RepositoryInstance.TestModeConfiguration = null;

            ViewModelInstance.Load();

            Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.");

            Assert.AreEqual<int>(0, ViewModelInstance.Stories.Items.Count, "Story count at start of test was wrong.");

            // cause the story repository to refresh and throw a refresh event

            EnqueueCallback(() =>
            RepositoryInstance.RefreshStories("2"));

            EnqueueDelay(3000);

            EnqueueCallback(() => 
            Assert.AreEqual<int>(10, ViewModelInstance.Stories.Items.Count, "Story collection should be populated."));

            EnqueueTestComplete();
        }

        [TestMethod]
        public void SelectItem_TriggersUpdatedEvent_UpdatesSelectedItemId()
        {
            // force the repository to write the test data to cache
            RepositoryInstance.RefreshStories("2");

            ViewModelInstance.Load();

            Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.");

            Assert.AreEqual<int>(10, ViewModelInstance.Stories.Items.Count, "Story count was wrong.");

            Assert.IsNull(ViewModelInstance.Stories.SelectedItem, "Nothing should be selected yet.");

            Assert.AreEqual<string>(null, ViewModelInstance.CurrentStoryId,
                "Current story id value was wrong before selection.");

            var expectedSelection = ViewModelInstance.Stories.Items[0];

            var tester = new StoryListViewModelEventTester(ViewModelInstance);

            expectedSelection.IsSelected = true;

            Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.");

            Assert.AreEqual<string>(expectedSelection.Id, ViewModelInstance.CurrentStoryId,
                "Current story id value was wrong.");
        }


        [TestMethod]
        public void SelectItem_SavesCurrentStoryIdToIsolatedStorage()
        {
            // force the repository to write the test data to cache
            RepositoryInstance.RefreshStories("2");
            ViewModelInstance.Load();

            Assert.IsNotNull(ViewModelInstance.Stories, "Stories was null.");

            SelectStoryAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0);
            SelectStoryAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 1);
        }

        [TestMethod]
        public void WhenLoadMethodIsCalledCurrentProgramIdAndTitleIsLoaded()
        {
            // program info is loaded via ProgramRepositoryFake
            ViewModelInstance.Load();

            UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId");
            UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramTitle, "CurrentProgramTitle");
        }

        [TestMethod]
        public void WhenLoadMethodIsCalledCurrentStoryIdIsLoaded()
        {
            // force the repository to write the test data to cache
            RepositoryInstance.RefreshStories("2");

            ViewModelInstance.Load();

            Assert.IsNull(ViewModelInstance.CurrentStoryId, "CurrentStoryId should be null.");
            
            SelectStoryAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0);

            var expectedCurrentStoryId = ViewModelInstance.CurrentStoryId;

            m_ViewModelInstance = null;

            ViewModelInstance.Load();

            Assert.AreEqual<string>(expectedCurrentStoryId, ViewModelInstance.CurrentStoryId,
                "CurrentStoryId did not match expected value after call to Load().");
        }

        private void SelectStoryAndAssertIdIsSavedToIsolatedStorage(IStoryListViewModel viewModel, int index)
        {
            var selectThis = viewModel.Stories.Items[index];

            selectThis.IsSelected = true;

            var repository = ObjectFactoryInstance.Create<IStoryRepository>();

            var currentStoryId = repository.GetCurrentStoryId();

            Assert.AreEqual<string>(selectThis.Id, currentStoryId, "Story Id did not match what was stored.");
        }

        private void AssertStory(
            IStoryListViewModel instance,
            int programIndex,
            string expectedTitle, string expectedId)
        {
            var actual = instance.Stories.Items[programIndex];

            Assert.AreEqual<string>(expectedTitle, actual.Text, "Text");
            Assert.AreEqual<string>(expectedId,
                actual.Id,
                "Id");
        }
    }
}
